@@ -165,8 +165,9 @@ public class LocationSharingFragment extends Fragment {
 
         Context ctx = requireContext();
         IConfigurationProvider configuration = Configuration.getInstance();
-        configuration.load(ctx, PreferenceManager.getDefaultSharedPreferences(ctx));
         configuration.setOsmdroidBasePath(new File(ctx.getCacheDir(), "osm"));
+        configuration.load(ctx, PreferenceManager.getDefaultSharedPreferences(ctx));
+        configuration.setUserAgentValue("net.jami.android");
         configuration.setMapViewHardwareAccelerated(true);
 
         bubbleSize = ctx.getResources().getDimensionPixelSize(R.dimen.location_sharing_avatar_size);
@@ -294,16 +295,18 @@ public class LocationSharingFragment extends Fragment {
         mDisposableBag.add(mServiceDisposableBag);
         mDisposableBag.add(mShowControlsSubject.subscribe(this::setShowControls));
         mDisposableBag.add(mIsSharingSubject.subscribe(this::setIsSharing));
-        mDisposableBag.add(mShowMapSubject.subscribe(state -> {
-            Fragment p = getParentFragment();
-            if (p instanceof ConversationFragment) {
-                ConversationFragment parent = (ConversationFragment) p;
-                if (state == MapState.FULL)
-                    parent.openLocationSharing();
-                else
-                    parent.closeLocationSharing(state == MapState.MINI);
-            }
-        }));
+        mDisposableBag.add(mShowMapSubject
+                .subscribeOn(AndroidSchedulers.mainThread())
+                .subscribe(state -> {
+                    Fragment p = getParentFragment();
+                    if (p instanceof ConversationFragment) {
+                        ConversationFragment parent = (ConversationFragment) p;
+                        if (state == MapState.FULL)
+                            parent.openLocationSharing();
+                        else
+                            parent.closeLocationSharing(state == MapState.MINI);
+                    }
+                }));
         mDisposableBag.add(mIsContactSharingSubject
                 .distinctUntilChanged()
                 .observeOn(AndroidSchedulers.mainThread())
