@@ -34,9 +34,9 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Random;
-import java.util.Timer;
-import java.util.TimerTask;
 import java.util.concurrent.ScheduledExecutorService;
+import java.util.concurrent.ScheduledFuture;
+import java.util.concurrent.TimeUnit;
 
 import javax.inject.Inject;
 import javax.inject.Named;
@@ -114,7 +114,6 @@ public class AccountService {
 
     private final HashMap<Long, DataTransfer> mDataTransfers = new HashMap<>();
     private DataTransfer mStartingTransfer = null;
-    private Timer mTransferRefreshTimer = null;
 
     private final BehaviorSubject<List<Account>> accountsSubject = BehaviorSubject.create();
     private final Subject<Account> accountSubject = PublishSubject.create();
@@ -584,7 +583,7 @@ public class AccountService {
 
     private Single<Account> loadAccountProfile(Account account) {
         if (account.getProfile() == null)
-            return VCardUtils.loadLocalProfileFromDisk(mDeviceRuntimeService.provideFilesDir(), account.getAccountID())
+            return VCardUtils.loadLocalProfileFromDiskWithDefault(mDeviceRuntimeService.provideFilesDir(), account.getAccountID())
                     .subscribeOn(Schedulers.io())
                     .map(vCard -> {
                         account.setProfile(vCard);
@@ -644,7 +643,7 @@ public class AccountService {
                         }
                         i++;
                     }
-                });
+                }, e -> Log.w(TAG, "Not sending empty profile", e));
     }
 
     public void setMessageDisplayed(String accountId, String contactId, String messageId) {
@@ -1164,10 +1163,9 @@ public class AccountService {
     public void sendTrustRequest(final String accountId, final String to, final Blob message) {
         Log.i(TAG, "sendTrustRequest() " + accountId + " " + to);
         handleTrustRequest(accountId, new Uri(to).getUri(), null, ContactType.ADDED);
-        mExecutor.execute(() -> Ringservice.sendTrustRequest(accountId, to, message));
+        mExecutor.execute(() -> Ringservice.sendTrustRequest(accountId, to, message == null ? new Blob() : message));
     }
 
-
     /**
      * Add a new contact for the account Id on the Daemon
      */
@@ -1519,8 +1517,9 @@ public class AccountService {
         mExecutor.execute(() -> Ringservice.cancelDataTransfer(dataTransferId));
     }
 
-    private class DataTransferRefreshTask extends TimerTask {
+    private class DataTransferRefreshTask implements Runnable {
         private final DataTransfer mToUpdate;
+        public ScheduledFuture<?> scheduledTask;
 
         DataTransferRefreshTask(DataTransfer t) {
             mToUpdate = t;
@@ -1528,13 +1527,12 @@ public class AccountService {
 
         @Override
         public void run() {
-            Interaction.InteractionStatus transferStatus;
             synchronized (mToUpdate) {
-                transferStatus = mToUpdate.getStatus();
-                if (transferStatus == Interaction.InteractionStatus.TRANSFER_ONGOING) {
+                if (mToUpdate.getStatus() == Interaction.InteractionStatus.TRANSFER_ONGOING) {
                     dataTransferEvent(mToUpdate.getDaemonId(), 5);
                 } else {
-                    cancel();
+                    scheduledTask.cancel(false);
+                    scheduledTask = null;
                 }
             }
         }
@@ -1569,12 +1567,10 @@ public class AccountService {
             InteractionStatus oldState = transfer.getStatus();
             if (oldState != transferStatus) {
                 if (transferStatus == Interaction.InteractionStatus.TRANSFER_ONGOING) {
-                    if (mTransferRefreshTimer == null)
-                        mTransferRefreshTimer = new Timer();
-                    mTransferRefreshTimer.scheduleAtFixedRate(
-                            new DataTransferRefreshTask(transfer),
+                    DataTransferRefreshTask task = new DataTransferRefreshTask(transfer);
+                    task.scheduledTask = mExecutor.scheduleAtFixedRate(task,
                             DATA_TRANSFER_REFRESH_PERIOD,
-                            DATA_TRANSFER_REFRESH_PERIOD);
+                            DATA_TRANSFER_REFRESH_PERIOD, TimeUnit.MILLISECONDS);
                 } else if (transferStatus.isError()) {
                     if (!transfer.isOutgoing()) {
                         File tmpPath = mDeviceRuntimeService.getTemporaryPath(transfer.getPeerId(), transfer.getStoragePath());
