@@ -24,7 +24,7 @@
 #include <regex>
 #include <stdexcept>
 
-//Manager
+// Manager
 #include "manager.h"
 #include "preferences.h"
 
@@ -36,46 +36,64 @@ extern "C" {
 #include <msgpack.hpp>
 
 #if defined(__arm__)
-  #if defined(__ARM_ARCH_7A__)
-    #define ABI "armeabi-v7a"
-  #else
-   #define ABI "armeabi"
-  #endif
-#elif defined(__i386__)
-   #define ABI "x86"
-#elif defined(__x86_64__)
-   #define ABI "x86_64"
-#elif defined(__mips64)  /* mips64el-* toolchain defines __mips__ too */
-   #define ABI "mips64"
-#elif defined(__mips__)
-   #define ABI "mips"
-#elif defined(__aarch64__)
-   #define ABI "arm64-v8a"
+#if defined(__ARM_ARCH_7A__)
+#define ABI "armeabi-v7a"
 #else
-   #define ABI "unknown"
+#define ABI "armeabi"
+#endif
+#elif defined(__i386__)
+#if __ANDROID__
+#define ABI "x86"
+#else
+#define ABI "x86-linux-gnu"
+#endif
+#elif defined(__x86_64__)
+#if __ANDROID__
+#define ABI "x86_64"
+#else
+#define ABI "x86_64-linux-gnu"
+#endif
+#elif defined(__mips64) /* mips64el-* toolchain defines __mips__ too */
+#define ABI "mips64"
+#elif defined(__mips__)
+#define ABI "mips"
+#elif defined(__aarch64__)
+#define ABI "arm64-v8a"
+#elif defined(WIN32)
+#define ABI "x64-windows"
+#else
+#define ABI "unknown"
 #endif
 
-#define	PLUGIN_ALREADY_INSTALLED	  100	/* Plugin already installed with the same version */
-#define	PLUGIN_OLD_VERSION	  200	/* Plugin already installed with a newer version */
+#define PLUGIN_ALREADY_INSTALLED 100 /* Plugin already installed with the same version */
+#define PLUGIN_OLD_VERSION       200 /* Plugin already installed with a newer version */
+
+#ifdef WIN32
+#define LIB_TYPE   ".dll"
+#define LIB_PREFIX ""
+#else
+#define LIB_TYPE   ".so"
+#define LIB_PREFIX "lib"
+#endif
 
 namespace jami {
 
-std::map<std::string, std::string> checkManifestJsonContentValidity(const Json::Value& root) {
+std::map<std::string, std::string>
+checkManifestJsonContentValidity(const Json::Value& root)
+{
     std::string name = root.get("name", "").asString();
     std::string description = root.get("description", "").asString();
     std::string version = root.get("version", "").asString();
-    if(!name.empty() || !version.empty()){
-        return {
-            {"name", name},
-            {"description", description},
-            {"version", version}
-        };
+    if (!name.empty() || !version.empty()) {
+        return {{"name", name}, {"description", description}, {"version", version}};
     } else {
         throw std::runtime_error("plugin manifest file: bad format");
     }
 }
 
-std::map<std::string, std::string> checkManifestValidity(std::istream& stream) {
+std::map<std::string, std::string>
+checkManifestValidity(std::istream& stream)
+{
     Json::Value root;
     Json::CharReaderBuilder rbuilder;
     rbuilder["collectComments"] = false;
@@ -83,76 +101,82 @@ std::map<std::string, std::string> checkManifestValidity(std::istream& stream) {
 
     bool ok = Json::parseFromStream(rbuilder, stream, &root, &errs);
 
-    if(ok) {
+    if (ok) {
         return checkManifestJsonContentValidity(root);
-    } else{
+    } else {
         throw std::runtime_error("failed to parse the plugin manifest file");
     }
 }
 
-std::map<std::string, std::string> checkManifestValidity(const std::vector<uint8_t>& vec) {
+std::map<std::string, std::string>
+checkManifestValidity(const std::vector<uint8_t>& vec)
+{
     Json::Value root;
-    std::unique_ptr<Json::CharReader> json_Reader(Json::CharReaderBuilder{}.newCharReader());
+    std::unique_ptr<Json::CharReader> json_Reader(Json::CharReaderBuilder {}.newCharReader());
     std::string errs;
 
-    bool ok = json_Reader->parse(reinterpret_cast<const char *>(vec.data()),
-                                 reinterpret_cast<const char *>(vec.data()+vec.size()),
-                                 &root,&errs);
+    bool ok = json_Reader->parse(reinterpret_cast<const char*>(vec.data()),
+                                 reinterpret_cast<const char*>(vec.data() + vec.size()),
+                                 &root,
+                                 &errs);
 
-    if(ok) {
+    if (ok) {
         return checkManifestJsonContentValidity(root);
-    } else{
+    } else {
         throw std::runtime_error("failed to parse the plugin manifest file");
     }
 }
 
 static const std::regex DATA_REGEX("^data" DIR_SEPARATOR_STR_ESC ".+");
-static const std::regex SO_REGEX("([a-zA-Z0-9]+(?:[_-]?[a-zA-Z0-9]+)*)" DIR_SEPARATOR_STR_ESC "([a-zA-Z0-9_-]+\\.(so|dll).*)");
+static const std::regex SO_REGEX("([a-zA-Z0-9]+(?:[_-]?[a-zA-Z0-9]+)*)" DIR_SEPARATOR_STR_ESC
+                                 "([a-zA-Z0-9_-]+\\.(so|dll|lib).*)");
 
-std::pair<bool,const std::string>
+std::pair<bool, const std::string>
 uncompressJplFunction(const std::string& relativeFileName)
 {
     std::smatch match;
-    if (relativeFileName == "manifest.json" || std::regex_match(relativeFileName, DATA_REGEX)){
+    if (relativeFileName == "manifest.json" || std::regex_match(relativeFileName, DATA_REGEX)) {
         return std::make_pair(true, relativeFileName);
     } else if (regex_search(relativeFileName, match, SO_REGEX) == true) {
         if (match.str(1) == ABI) {
             return std::make_pair(true, match.str(2));
         }
     }
-    return std::make_pair(false, std::string{""});
+    return std::make_pair(false, std::string {""});
 }
 
-std::string convertArrayToString(const Json::Value& jsonArray)
+std::string
+convertArrayToString(const Json::Value& jsonArray)
 {
     std::string stringArray = "[";
 
-    for(int i=0; i< static_cast<int>(jsonArray.size()) - 1; i++) {
-        if(jsonArray[i].isString()) {
-            stringArray+=jsonArray[i].asString()+",";
-        } else if(jsonArray[i].isArray()) {
-            stringArray+=convertArrayToString(jsonArray[i])+",";
+    for (int i = 0; i < static_cast<int>(jsonArray.size()) - 1; i++) {
+        if (jsonArray[i].isString()) {
+            stringArray += jsonArray[i].asString() + ",";
+        } else if (jsonArray[i].isArray()) {
+            stringArray += convertArrayToString(jsonArray[i]) + ",";
         }
     }
 
     int lastIndex = static_cast<int>(jsonArray.size()) - 1;
-    if(jsonArray[lastIndex].isString()) {
-        stringArray+=jsonArray[lastIndex].asString();
+    if (jsonArray[lastIndex].isString()) {
+        stringArray += jsonArray[lastIndex].asString();
     }
 
-    stringArray+="]";
+    stringArray += "]";
 
     return stringArray;
 }
 
-std::map<std::string, std::string> parsePreferenceConfig(const Json::Value &jsonPreference, const std::string &type)
+std::map<std::string, std::string>
+parsePreferenceConfig(const Json::Value& jsonPreference, const std::string& type)
 {
     std::map<std::string, std::string> preferenceMap;
     const auto& members = jsonPreference.getMemberNames();
     // Insert other fields
-    for(const auto& member : members) {
+    for (const auto& member : members) {
         const Json::Value& value = jsonPreference[member];
-        if(value.isString()) {
+        if (value.isString()) {
             preferenceMap.emplace(member, jsonPreference[member].asString());
         } else if (value.isArray()) {
             preferenceMap.emplace(member, convertArrayToString(jsonPreference[member]));
@@ -161,7 +185,8 @@ std::map<std::string, std::string> parsePreferenceConfig(const Json::Value &json
     return preferenceMap;
 }
 
-std::map<std::string, std::string> JamiPluginManager::getPluginDetails(const std::string &rootPath)
+std::map<std::string, std::string>
+JamiPluginManager::getPluginDetails(const std::string& rootPath)
 {
     auto detailsIt = pluginDetailsMap_.find(rootPath);
     if (detailsIt != pluginDetailsMap_.end()) {
@@ -169,56 +194,63 @@ std::map<std::string, std::string> JamiPluginManager::getPluginDetails(const std
     }
 
     std::map<std::string, std::string> details = parseManifestFile(manifestPath(rootPath));
-    if (!details.empty())
-    {
+    if (!details.empty()) {
         details["iconPath"] = rootPath + DIR_SEPARATOR_CH + "data" + DIR_SEPARATOR_CH + "icon.png";
-        details["soPath"] = rootPath + DIR_SEPARATOR_CH + "lib" + details["name"] + ".so";
+        details["soPath"] = rootPath + DIR_SEPARATOR_CH + LIB_PREFIX + details["name"] + LIB_TYPE;
         detailsIt = pluginDetailsMap_.emplace(rootPath, std::move(details)).first;
         return detailsIt->second;
     }
     return {};
 }
 
-std::vector<std::string> JamiPluginManager::listAvailablePlugins()
+std::vector<std::string>
+JamiPluginManager::listAvailablePlugins()
 {
     std::string pluginsPath = fileutils::get_data_dir() + DIR_SEPARATOR_CH + "plugins";
     std::vector<std::string> pluginsPaths = fileutils::readDirectory(pluginsPath);
-    std::for_each(pluginsPaths.begin(), pluginsPaths.end(),
-                  [&pluginsPath](std::string& x){ x = pluginsPath + DIR_SEPARATOR_CH + x;});
-    auto predicate = [this](std::string path){ return !checkPluginValidity(path);};
-    auto returnIterator = std::remove_if(pluginsPaths.begin(),pluginsPaths.end(),predicate);
-    pluginsPaths.erase(returnIterator,std::end(pluginsPaths));
+    std::for_each(pluginsPaths.begin(), pluginsPaths.end(), [&pluginsPath](std::string& x) {
+        x = pluginsPath + DIR_SEPARATOR_CH + x;
+    });
+    auto predicate = [this](std::string path) {
+        return !checkPluginValidity(path);
+    };
+    auto returnIterator = std::remove_if(pluginsPaths.begin(), pluginsPaths.end(), predicate);
+    pluginsPaths.erase(returnIterator, std::end(pluginsPaths));
     return pluginsPaths;
 }
 
-int JamiPluginManager::installPlugin(const std::string &jplPath, bool force)
+int
+JamiPluginManager::installPlugin(const std::string& jplPath, bool force)
 {
-    int r{0};
-    if(fileutils::isFile(jplPath)) {
-        try{
+    int r {0};
+    if (fileutils::isFile(jplPath)) {
+        try {
             auto manifestMap = readPluginManifestFromArchive(jplPath);
             std::string name = manifestMap["name"];
+            if (name.empty())
+                return 0;
             std::string version = manifestMap["version"];
-            const std::string destinationDir{fileutils::get_data_dir()
-                                             + DIR_SEPARATOR_CH + "plugins"
-                                             + DIR_SEPARATOR_CH + name};
+            const std::string destinationDir {fileutils::get_data_dir() + DIR_SEPARATOR_CH
+                                              + "plugins" + DIR_SEPARATOR_CH + name};
             // Find if there is an existing version of this plugin
             const auto alreadyInstalledManifestMap = parseManifestFile(manifestPath(destinationDir));
 
             if (!alreadyInstalledManifestMap.empty()) {
                 if (force) {
                     r = uninstallPlugin(destinationDir);
-                    if(r == 0) {
+                    if (r == 0) {
                         archiver::uncompressArchive(jplPath, destinationDir, uncompressJplFunction);
                     }
                 } else {
                     std::string installedVersion = alreadyInstalledManifestMap.at("version");
                     if (version > installedVersion) {
                         r = uninstallPlugin(destinationDir);
-                        if(r == 0) {
-                            archiver::uncompressArchive(jplPath, destinationDir, uncompressJplFunction);
+                        if (r == 0) {
+                            archiver::uncompressArchive(jplPath,
+                                                        destinationDir,
+                                                        uncompressJplFunction);
                         }
-                    } else if (version == installedVersion){
+                    } else if (version == installedVersion) {
                         // An error code of 100 to know that this version is the same as the one installed
                         r = PLUGIN_ALREADY_INSTALLED;
                     } else {
@@ -229,27 +261,24 @@ int JamiPluginManager::installPlugin(const std::string &jplPath, bool force)
             } else {
                 archiver::uncompressArchive(jplPath, destinationDir, uncompressJplFunction);
             }
-        } catch(const std::exception& e) {
+        } catch (const std::exception& e) {
             JAMI_ERR() << e.what();
         }
     }
     return r;
 }
 
-int JamiPluginManager::uninstallPlugin(const std::string &rootPath)
+int
+JamiPluginManager::uninstallPlugin(const std::string& rootPath)
 {
-    if (checkPluginValidity(rootPath))
-    {
+    if (checkPluginValidity(rootPath)) {
         auto detailsIt = pluginDetailsMap_.find(rootPath);
-        if (detailsIt != pluginDetailsMap_.end())
-        {
+        if (detailsIt != pluginDetailsMap_.end()) {
             bool loaded = pm_.checkLoadedPlugin(detailsIt->second.at("soPath"));
-            if (loaded)
-            {
+            if (loaded) {
                 JAMI_INFO() << "PLUGIN: unloading before uninstall.";
                 bool status = unloadPlugin(rootPath);
-                if (!status)
-                {
+                if (!status) {
                     JAMI_INFO() << "PLUGIN: could not unload, not performing uninstall.";
                     return -1;
                 }
@@ -263,11 +292,11 @@ int JamiPluginManager::uninstallPlugin(const std::string &rootPath)
     }
 }
 
-bool JamiPluginManager::loadPlugin(const std::string &rootPath)
+bool
+JamiPluginManager::loadPlugin(const std::string& rootPath)
 {
 #ifdef ENABLE_PLUGIN
-    try
-    {
+    try {
         bool status = pm_.load(getPluginDetails(rootPath).at("soPath"));
         JAMI_INFO() << "PLUGIN: load status - " << status;
 
@@ -275,8 +304,7 @@ bool JamiPluginManager::loadPlugin(const std::string &rootPath)
         jami::Manager::instance().saveConfig();
         return status;
 
-    } catch(const std::exception& e) 
-    {
+    } catch (const std::exception& e) {
         JAMI_ERR() << e.what();
         return false;
     }
@@ -284,11 +312,11 @@ bool JamiPluginManager::loadPlugin(const std::string &rootPath)
     return false;
 }
 
-bool JamiPluginManager::unloadPlugin(const std::string &rootPath)
+bool
+JamiPluginManager::unloadPlugin(const std::string& rootPath)
 {
 #ifdef ENABLE_PLUGIN
-    try
-    {
+    try {
         bool status = pm_.unload(getPluginDetails(rootPath).at("soPath"));
         JAMI_INFO() << "PLUGIN: unload status - " << status;
 
@@ -296,8 +324,7 @@ bool JamiPluginManager::unloadPlugin(const std::string &rootPath)
         jami::Manager::instance().saveConfig();
 
         return status;
-    } catch(const std::exception& e)
-    {
+    } catch (const std::exception& e) {
         JAMI_ERR() << e.what();
         return false;
     }
@@ -305,16 +332,16 @@ bool JamiPluginManager::unloadPlugin(const std::string &rootPath)
     return false;
 }
 
-void JamiPluginManager::togglePlugin(const std::string &rootPath, bool toggle)
+void
+JamiPluginManager::togglePlugin(const std::string& rootPath, bool toggle)
 {
-    try
-    {
+    try {
         std::string soPath = getPluginDetails(rootPath).at("soPath");
         // remove the previous plugin object if it was registered
         pm_.destroyPluginComponents(soPath);
         // If toggle, register a new instance of the plugin
         // function
-        if(toggle){
+        if (toggle) {
             pm_.callPluginInitFunction(soPath);
         }
     } catch (const std::exception& e) {
@@ -322,19 +349,21 @@ void JamiPluginManager::togglePlugin(const std::string &rootPath, bool toggle)
     }
 }
 
-std::vector<std::string> JamiPluginManager::listLoadedPlugins() const
+std::vector<std::string>
+JamiPluginManager::listLoadedPlugins() const
 {
     std::vector<std::string> loadedSoPlugins = pm_.listLoadedPlugins();
-    std::vector<std::string> loadedPlugins{};
+    std::vector<std::string> loadedPlugins {};
     loadedPlugins.reserve(loadedSoPlugins.size());
-    std::transform(loadedSoPlugins.begin(), loadedSoPlugins.end(), std::back_inserter(loadedPlugins),
-    [this](const std::string& soPath) {
-        return getRootPathFromSoPath(soPath);
-    });
+    std::transform(loadedSoPlugins.begin(),
+                   loadedSoPlugins.end(),
+                   std::back_inserter(loadedPlugins),
+                   [this](const std::string& soPath) { return getRootPathFromSoPath(soPath); });
     return loadedPlugins;
 }
 
-std::vector<std::map<std::string, std::string> > JamiPluginManager::getPluginPreferences(const std::string &rootPath)
+std::vector<std::map<std::string, std::string>>
+JamiPluginManager::getPluginPreferences(const std::string& rootPath)
 {
     const std::string preferenceFilePath = getPreferencesConfigFilePath(rootPath);
     std::ifstream file(preferenceFilePath);
@@ -344,22 +373,23 @@ std::vector<std::map<std::string, std::string> > JamiPluginManager::getPluginPre
     std::string errs;
     std::set<std::string> keys;
     std::vector<std::map<std::string, std::string>> preferences;
-    if(file) {
+    if (file) {
         bool ok = Json::parseFromStream(rbuilder, file, &root, &errs);
-        if(ok && root.isArray()) {
-            for(int i=0; i< static_cast<int>(root.size()); i++) {
+        if (ok && root.isArray()) {
+            for (int i = 0; i < static_cast<int>(root.size()); i++) {
                 const Json::Value jsonPreference = root[i];
                 std::string category = jsonPreference.get("category", "NoCategory").asString();
                 std::string type = jsonPreference.get("type", "None").asString();
                 std::string key = jsonPreference.get("key", "None").asString();
-                if(type != "None" && key != "None") {
-                    if(keys.find(key) == keys.end()) {
-                       const auto& preferenceAttributes = parsePreferenceConfig(jsonPreference, type);
-                       // If the parsing of the attributes was successful, commit the map and the key
-                       if(!preferenceAttributes.empty()) {
-                           preferences.push_back(std::move(preferenceAttributes));
-                           keys.insert(key);
-                       }
+                if (type != "None" && key != "None") {
+                    if (keys.find(key) == keys.end()) {
+                        const auto& preferenceAttributes = parsePreferenceConfig(jsonPreference,
+                                                                                 type);
+                        // If the parsing of the attributes was successful, commit the map and the key
+                        if (!preferenceAttributes.empty()) {
+                            preferences.push_back(std::move(preferenceAttributes));
+                            keys.insert(key);
+                        }
                     }
                 }
             }
@@ -372,11 +402,12 @@ std::vector<std::map<std::string, std::string> > JamiPluginManager::getPluginPre
     return preferences;
 }
 
-std::map<std::string, std::string> JamiPluginManager::getPluginUserPreferencesValuesMap(const std::string &rootPath)
+std::map<std::string, std::string>
+JamiPluginManager::getPluginUserPreferencesValuesMap(const std::string& rootPath)
 {
     const std::string preferencesValuesFilePath = pluginPreferencesValuesFilePath(rootPath);
     std::ifstream file(preferencesValuesFilePath, std::ios::binary);
-    std::map<std::string, std::string>  rmap;
+    std::map<std::string, std::string> rmap;
 
     // If file is accessible
     if (file.good()) {
@@ -406,19 +437,23 @@ std::map<std::string, std::string> JamiPluginManager::getPluginUserPreferencesVa
     return rmap;
 }
 
-bool JamiPluginManager::setPluginPreference(const std::string &rootPath, const std::string &key, const std::string &value)
+bool
+JamiPluginManager::setPluginPreference(const std::string& rootPath,
+                                       const std::string& key,
+                                       const std::string& value)
 {
     bool returnValue = false;
-    std::map<std::string, std::string> pluginUserPreferencesMap = getPluginUserPreferencesValuesMap(rootPath);
-    std::map<std::string, std::string> pluginPreferencesMap = getPluginPreferencesValuesMap(rootPath);
+    std::map<std::string, std::string> pluginUserPreferencesMap = getPluginUserPreferencesValuesMap(
+        rootPath);
+    std::map<std::string, std::string> pluginPreferencesMap = getPluginPreferencesValuesMap(
+        rootPath);
 
     auto find = pluginPreferencesMap.find(key);
-    if (find != pluginPreferencesMap.end())
-    {
+    if (find != pluginPreferencesMap.end()) {
         pluginUserPreferencesMap[key] = value;
         const std::string preferencesValuesFilePath = pluginPreferencesValuesFilePath(rootPath);
         std::ofstream fs(preferencesValuesFilePath, std::ios::binary);
-        if(!fs.good()) {
+        if (!fs.good()) {
             return false;
         }
         try {
@@ -434,11 +469,12 @@ bool JamiPluginManager::setPluginPreference(const std::string &rootPath, const s
     return returnValue;
 }
 
-std::map<std::string, std::string> JamiPluginManager::getPluginPreferencesValuesMap(const std::string &rootPath)
+std::map<std::string, std::string>
+JamiPluginManager::getPluginPreferencesValuesMap(const std::string& rootPath)
 {
-    std::map<std::string, std::string>  rmap;
+    std::map<std::string, std::string> rmap;
 
-    std::vector<std::map<std::string,std::string>> preferences = getPluginPreferences(rootPath);
+    std::vector<std::map<std::string, std::string>> preferences = getPluginPreferences(rootPath);
     for (size_t i = 0; i < preferences.size(); i++) {
         rmap[preferences[i]["key"]] = preferences[i]["defaultValue"];
     }
@@ -449,14 +485,15 @@ std::map<std::string, std::string> JamiPluginManager::getPluginPreferencesValues
     return rmap;
 }
 
-bool JamiPluginManager::resetPluginPreferencesValuesMap(const std::string &rootPath)
+bool
+JamiPluginManager::resetPluginPreferencesValuesMap(const std::string& rootPath)
 {
     bool returnValue = true;
-    std::map<std::string, std::string> pluginPreferencesMap{};
+    std::map<std::string, std::string> pluginPreferencesMap {};
 
     const std::string preferencesValuesFilePath = pluginPreferencesValuesFilePath(rootPath);
     std::ofstream fs(preferencesValuesFilePath, std::ios::binary);
-    if(!fs.good()) {
+    if (!fs.good()) {
         return false;
     }
     try {
@@ -470,20 +507,22 @@ bool JamiPluginManager::resetPluginPreferencesValuesMap(const std::string &rootP
     return returnValue;
 }
 
-std::map<std::string, std::string> JamiPluginManager::readPluginManifestFromArchive(const std::string &jplPath)
+std::map<std::string, std::string>
+JamiPluginManager::readPluginManifestFromArchive(const std::string& jplPath)
 {
     try {
-        return checkManifestValidity(archiver::readFileFromArchive(jplPath,"manifest.json"));
+        return checkManifestValidity(archiver::readFileFromArchive(jplPath, "manifest.json"));
     } catch (const std::exception& e) {
         JAMI_ERR() << e.what();
     }
     return {};
 }
 
-std::map<std::string, std::string> JamiPluginManager::parseManifestFile(const std::string &manifestFilePath)
+std::map<std::string, std::string>
+JamiPluginManager::parseManifestFile(const std::string& manifestFilePath)
 {
     std::ifstream file(manifestFilePath);
-    if(file) {
+    if (file) {
         try {
             return checkManifestValidity(file);
         } catch (const std::exception& e) {
@@ -493,13 +532,13 @@ std::map<std::string, std::string> JamiPluginManager::parseManifestFile(const st
     return {};
 }
 
-void JamiPluginManager::registerServices()
+void
+JamiPluginManager::registerServices()
 {
     // Register pluginPreferences
     pm_.registerService("getPluginPreferences", [this](const DLPlugin* plugin, void* data) {
         auto ppp = static_cast<std::map<std::string, std::string>*>(data);
-        *ppp = getPluginPreferencesValuesMap(
-            getRootPathFromSoPath(plugin->getPath()));
+        *ppp = getPluginPreferencesValuesMap(getRootPathFromSoPath(plugin->getPath()));
         return 0;
     });
 
@@ -510,4 +549,4 @@ void JamiPluginManager::registerServices()
     });
 }
 
-}
+} // namespace jami
